"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
/**
 * the New code to inject in Page
 */
class JQueryAble {
    jQuery(selector) {
        return new Proxy(new PProxyApi(this, selector, ''), handlerRoot);
    }
}
exports.JQueryAble = JQueryAble;
/**
 * random variable name generator
 */
exports.randName = () => {
    let c1 = Math.floor((Math.random() * 25));
    return 'abcdefghijklmnopqrstuvwxyz'[c1] + '_' + Number(String(Math.random()).substr(2)).toString(36);
};
/**
 * internal used isString funtion
 */
function isString(obj) {
    return typeof obj === 'string' || obj instanceof String;
}
const handlerRoot = {
    get(target, p, receiver) {
        // console.log(`get(${p.toString()})`, receiver);
        if (typeof p == 'symbol')
            return target[p];
        let key = p.toString();
        switch (key) {
            // passthru
            case 'toString':
            case 'valueOf':
                return target[p];
            case 'code':
            case 'selector':
            case 'page':
                return target[p];
            /**
             * if exec() is forgoten, act as a promise.
             */
            case 'then': // start exec Promise
                return (...args) => {
                    const lastExec = target.exec(true, false);
                    return lastExec.then(...args);
                };
            case 'exec': // start exec Promise
                return (...args) => {
                    return target.exec(true, false);
                };
            case 'pojo': // start exec Promise
                return (...args) => {
                    // console.log('csll exec ', args);
                    return target.exec(true, true);
                };
        }
        return (...args) => {
            args = args.map((arg) => {
                if (isString(arg)) {
                    return JSON.stringify(arg);
                }
                if (typeof arg === 'function') {
                    return arg.toString().replace(/jQuery\(/g, `${jQueryName}(`);
                }
                // is POJO
                return JSON.stringify(arg);
            });
            let newCode = `${target.code}.${key}(${args.join(',')})`;
            if (args.length === 0) {
                switch (key) {
                    case 'text':
                    case 'html':
                    case 'val':
                    case 'css':
                        const tmp = new PProxyApi(target.page, target.selector, newCode);
                        return tmp.exec(false, true);
                }
            }
            let child = new PProxyApi(target.page, target.selector, newCode);
            return new Proxy(child, handlerRoot);
        };
    }
};
/**
 * choose a tmp name once per Launch
 */
const jQueryName = exports.randName();
/**
 * jquery data storage
 */
let jQueryData = '';
/**
 * transparant jquery interface
 */
class PProxyApi {
    constructor(page, selector, code) {
        this.page = page;
        this.selector = selector;
        this.code = code;
    }
    toString() {
        return 'JQuery selector based on:' + this.page;
    }
    /**
     * @param toArray must be set to true is the object is a jQuery
     * @param isPOJO must be set to true if waiting for plain data, fals is waiting for HTMLElements
     */
    async exec(toArray, isPOJO) {
        let code = `${jQueryName}('${this.selector.replace(/'/g, "\\\'")}')`;
        code += this.code;
        if (toArray)
            code += `.toArray()`;
        let handle;
        const { page } = this;
        // console.log(code, "POJO is", isPOJO);
        try {
            try {
                handle = await page.evaluateHandle(code);
            }
            catch (e) {
                if (!jQueryData) {
                    // Sync call, do not want to force nodejs 10+ nor adding js-extra, not using a call back here
                    // nor adding a new Promise statement that would take as much space as this comment...
                    const jqData = fs_1.default.readFileSync(path_1.default.join(__dirname, '..', 'data', 'jquery-3.4.1.js'), { encoding: 'utf-8' });
                    jQueryData = '//# sourceURL=jquery.js\n' + jqData.replace('window.jQuery = window.$ = jQuery', `window.${jQueryName} = jQuery`);
                    // TODO add minify code.
                }
                if (~e.message.indexOf(`${jQueryName} is not defined`)) {
                    await page.evaluate(jQueryData); // define jQuery
                    handle = await page.evaluateHandle(code);
                }
                else {
                    throw e;
                }
            }
            if (isPOJO) {
                let value = await handle.jsonValue();
                await handle.dispose();
                return value;
            }
            const array = [];
            const properties = await handle.getProperties();
            for (const property of properties.values()) {
                const elementHandle = property.asElement();
                if (elementHandle)
                    array.push(elementHandle);
            }
            await handle.dispose();
            return array;
        }
        catch (e2) {
            console.error(`Exec: ${code}`);
            if (e2.message)
                e2.message = `exec: ${code}\n failed:${e2.message}`;
            throw e2;
        }
    }
}
